/****************************************************************************\
**  Title:        PCXCODE                                                   **
**  Purpose:      Decode/Encode a PCX image file.                           **
**  Version:      1.0                                                       **
**  Date:         May 1991                                                  **
**  Author:       James D. Murray, Anaheim, CA  USA                         **
**  C Compilers:  Borland C++ v2.0, Microsoft C v6.00a                      **
**                                                                          **
**  This code is an example of several functions that work with PCX image   **
**  files.  PCXCODE reads a PCX image file, decodes the image, data         **
**  re-encodes it, and writes it out to a second PCX file.  While this      **
**  program is only a working example, it can be used to possibly repair    **
**  PCX files that have been previously encoded incorrectly and fail to     **
**  display properly.                                                           
                **
**                                                                          **
**  Copyright (C) 1991 Graphics Software Labs.  All rights reserved.        **
\****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "endianio.h"
#include "pcx.h"


int
main(argc, argv)
int argc;           /* Number of command line arguments */
char *argv[];       /* Array of command line arguments  */
{
    register WORD i;                /* Loop counter                      */
    FILE         *fpPcxIn;          /* PCX image file input FILE stream  */
    FILE         *fpPcxOut;         /* PCX image file output FILE stream */
    PCXHEADER     pcxHead;          /* PCX image file header structure   */
    WORD          numOfColors;      /* Number of colors in the image     */
    WORD          xSize;            /* Width of image in pixels          */
    WORD          ySize;            /* Height of image in scan lines     */
    SHORT         scanLineLength;       /* Length of uncompressed scan line  */
    BYTE         *decodedBuffer;    /* Buffer holding decoded scan line  */
    BYTE         *encodedBuffer;    /* Buffer holding encoded scan line  */
    WORD          scanLineSize;     /* Size of encoded scan line         */
    SHORT         ret;              /* Return value holder               */

    puts("PCXCODE - Decode and recode a PCX image file (v1.00)\n");

    /* Check for proper number of command line arguments */
    if (argc < 3)
    {
        fputs("Usage: PCXHEAD input_filename.pcx output_filename.pcx\n\n",
          stderr);
        exit(-1);
    }

    /* Open the input PCX image file */
    if ((fpPcxIn = fopen(argv[1], "rb")) == (FILE *) NULL)
    {
        fprintf(stderr, "PCXHEAD: Cannot open input file %s\n", argv[1]);
        exit(-2);
    }

    /* Open the output PCX image file */
    if ((fpPcxOut = fopen(argv[2], "wb")) == (FILE *) NULL)
    {
        fprintf(stderr, "PCXHEAD: Cannot open output file %s\n", argv[2]);
        exit(-3);
    }

    /* Read the PCX image file header information */
    ReadPcxHeader(&pcxHead, fpPcxIn);

    /* Check for FILE stream error */
    if (ferror(fpPcxIn))
    {
        fputs("PCXHEAD: Error reading header information!\n", stderr);
        exit(-4);
    }

    /* Check the identification byte value */
    if (pcxHead.Id != 0x0A)
    {
        fprintf(stderr, "PCXHEAD: Not a PCX-format file!\n");
        exit(-5);
    }

    /* Calculate size of image in pixels and scan lines */
    xSize = pcxHead.Xmax - pcxHead.Xmin + 1;
    ySize = pcxHead.Ymax - pcxHead.Ymin + 1;

    /* Do some more calculations */
    numOfColors = (1 << (pcxHead.BitsPixelPlane * pcxHead.NumberOfPlanes));
    scanLineLength = pcxHead.NumberOfPlanes * pcxHead.BytesLinePlane;

    /* Display some PCX image file information */
    printf("Image Width  = %3d\nImage Length = %3d\n", xSize, ySize);
    printf("Number of Colors: %d\n", numOfColors);
    printf("Size of uncompressed scan line: %d\n\n", scanLineLength);

    /* Write out the PCX image file header */
    WritePcxHeader(&pcxHead, fpPcxOut);

    /* Create an uncompressed scanline buffer */
    if ((decodedBuffer = (BYTE *) malloc(scanLineLength)) == NULL)
    {
        fputs("PCXCODE: Cannot allocate scan line buffer.", stderr);
        exit(-6);
    }

    /* Create a compressed scanline buffer */
    if ((encodedBuffer = (BYTE *) malloc(2048)) == NULL)
    {
        fputs("PCXCODE: Cannot allocate scan line buffer.", stderr);
        exit(-7);
    }

    /*
    ** Read an encoded scan line, decode it, reencode it, and write it to
    ** the output file stream.  Continue until all scan lines are read.
    */
    for (i = 0; i < ySize; i++)
    {
        printf("Scan Line %03d\r", i);

        /* Read scan line and decode it */
        if ((ret = PcxDecodeScanLine(decodedBuffer, scanLineLength, fpPcxIn))
          != scanLineLength)
        {
            printf("PCXCODE: Bad scan line length (was %d should be %d)\n",
              ret, scanLineLength);
        }

        /* Encode scan line */
        scanLineSize =
          PcxEncodeScanLine(decodedBuffer, encodedBuffer, scanLineLength);

        /* Write scan line to output FILE stream */
        fwrite((BYTE *) encodedBuffer, sizeof(BYTE), scanLineSize, fpPcxOut);
    }

    /* If there is a VGA palette then read and append it to the output file */
    if (pcxHead.Version == 5)
    {
        fseek(fpPcxIn, -769L, SEEK_END);  /* Seek backwards from end of file */

        if (GetByte(fpPcxIn) == 0x0C)     /* VGA Palette ID value? */ 
        {
            PCXVGAPALETTE  PcxVgaPal;     /* PCX VGA palette structure */

            fputc(0x0c, fpPcxOut);

            /* Read the VGA palette */
            for (i = 0; i < sizeof(PcxVgaPal); i++)
                PcxVgaPal.VgaPalette[i] = GetByte(fpPcxIn);

            /* Write the VGA palette */
            for (i = 0; i < sizeof(PcxVgaPal); i++)
                PutByte(PcxVgaPal.VgaPalette[i], fpPcxOut);
        }
    }
    fclose(fpPcxIn);
    fclose(fpPcxOut);

    return(0);
}
}
